#ifndef _PURNETWORK_CNULL_SOCKET_H
#define _PURNETWORK_CNULL_SOCKET_H

#include "PureNetwork.h"
#include "ISocket.h"

// PurNetwork Null Socket Class

namespace pun
{
	namespace socket
	{
		class CNullSocket : public ISocket
		{
		public:

			// Constructor
			CNullSocket();

			// Destructor
			virtual ~CNullSocket();

			// Listen server.
			virtual RESULT listenServer(SckListen sckListen);

			// Connect server.
			virtual RESULT connectServer(SckConnect sckConnect);

			// Close socket.
			virtual void closeSocket();

			// Set Event Receiver
			virtual void setEventReceiver(IEventReceiver *pEventReceiver)
			{
				m_pEventReceiver = pEventReceiver;
			}

			// Get Connection
			virtual IConnection *getConnection()
			{
				return m_pConnection;
			}

			// Send Message
			// If Client, pReceiver = 0 means Server.
			virtual RESULT sendMessage(IConnection *pReceiver, core::CDataBuffer **ppDataBuffer, c32 nArraySize = 1);

			// is Connected ?
			virtual bool isConnected();

			// is Listened ?
			virtual bool isListened();

			// On Event
			virtual void onEvent(EventMsg eventMsg);

		protected:

			//
			// Logical events
			//

			// New Client!
			virtual void onNewClient(IConnection *pConnection);

			// Connection object
			IConnection *m_pConnection;

			// Event receiver
			IEventReceiver *m_pEventReceiver;

			// Connection list
			core::list<IConnection *> m_lstConnection;
		};
	};
};	

#endif