#include "PureNetwork.h"
#include "CWinsock2Connection.h"

#ifdef _INCLUDE_WINSOCK2

namespace pun
{
	namespace socket
	{
		// Constructor
		CWinsock2Connection::CWinsock2Connection()
		{

		}

		// Destructor
		CWinsock2Connection::~CWinsock2Connection()
		{
			// Close socket of connection.
			if(m_socket)
			{
				closesocket(m_socket);
				m_socket = INVALID_SOCKET;
			}
		}

		// Create socket
		RESULT CWinsock2Connection::createSocket(E_PROTOCOL_TYPE protocolType, E_ADDRESS_TYPE addressType, bool bOverlapped)
		{
			// Already initialized.
			if(ASSERT_LOG(m_socket != INVALID_SOCKET, "Socket has been already initialized."))
				return E_FAIL;

			u32 nFlags = 0;
			if(bOverlapped)
				nFlags |= WSA_FLAG_OVERLAPPED;

			c32 nAddressType;
			switch(addressType)
			{
			case EAT_IPV4:
				nAddressType = PF_INET;
				break;
			case EAT_IPV6:
				nAddressType = PF_INET6;
				break;
			default:
				nAddressType = PF_INET;
			}

			c32 nProtocolType;
			switch(protocolType)
			{
			case EPT_TCP:
				nProtocolType = SOCK_STREAM;
				break;
			case EPT_UDP:
				nProtocolType = SOCK_DGRAM;
			default:
				nProtocolType = SOCK_STREAM;
			}

			// Create socket
			m_socket = WSASocket(nAddressType, nProtocolType, 0, 0, 0, nFlags);
			if(ASSERT_LOG(m_socket != INVALID_SOCKET, "Failed to create socket."))
				return E_FAIL;

			// Set socket step.
			m_socketStep = ECST_CREATE;
			return S_OK;
		}

		// Set for using server
		RESULT CWinsock2Connection::setUseServer(SckListen sckListen)
		{
			// Bind
			if(m_socketStep == ECST_CREATE)
			{
				c32 nAddressFamily;
				
				// Construct Socket address.
				memset(&m_sockAddr, 0, sizeof(SOCKADDR_IN));
				m_sockAddr.sin_port = htons(sckListen.nPort);

				// Set address family 
				switch(sckListen.addressType)
				{
				case EAT_IPV4:
					nAddressFamily = PF_INET;
					break;
				case EAT_IPV6:
					nAddressFamily = PF_INET6;
					break;
				default:
					nAddressFamily = PF_INET;
				}
				m_sockAddr.sin_family = nAddressFamily;

				// Fixing
				if(sckListen.pAddress[0] != '\0')
				{
					m_sockAddr.sin_addr.s_addr = inet_addr(sckListen.pAddress);

					// Set address
					m_strAddress = sckListen.pAddress;
				}
				// Any
				else
				{
					m_sockAddr.sin_addr.s_addr = inet_addr(INADDR_ANY);

					// Set address
					m_strAddress = ANY_ADDRESS;
				}

				// Set port
				m_nPort = sckListen.nPort;

				if(ASSERT_LOG(bind(m_socket, (sockaddr *)&m_sockAddr, sizeof(SOCKADDR_IN)) != SOCKET_ERROR, "Failed to bind socket."))
					return E_FAIL;

				m_socketStep = ECST_BIND;
			}

			// Listen
			if(m_socketStep == ECST_BIND)
			{
				// Check parameter
				if(ASSERT_LOG(sckListen.nBacklog > 0, "Variable Backlog is wrong value."))
					return E_FAIL;

				if(ASSERT_LOG(listen(m_socket, sckListen.nBacklog) != SOCKET_ERROR, "Failed to listen socket."))
					return E_FAIL;

				m_socketStep = ECST_COMPLETE;
				m_conType = ECT_SERVER;
			}

			return S_OK;
		}

		// Set for using client
		RESULT CWinsock2Connection::setUseClient(SckConnect sckConnect)
		{
			// Bind
			if(m_socketStep == ECST_CREATE)
			{
				// Check parameter
				if(ASSERT_LOG(sckConnect.pAddress[0] != '\0', "Connect Address is wrong value!"))
					return E_FAIL;

				c32 nAddressFamily;

				// Construct Socket address.
				memset(&m_sockAddr, 0, sizeof(SOCKADDR_IN));
				m_sockAddr.sin_port = htons(sckConnect.nPort);

				// Set address family 
				switch(sckConnect.addressType)
				{
				case EAT_IPV4:
					nAddressFamily = PF_INET;
					break;
				case EAT_IPV6:
					nAddressFamily = PF_INET6;
					break;
				default:
					nAddressFamily = PF_INET;
				}
				m_sockAddr.sin_family = nAddressFamily;

				// Fixing
				if(sckConnect.pAddress[0] != '\0')
				{
					m_sockAddr.sin_addr.s_addr = inet_addr(sckConnect.pAddress);

					// Set address
					m_strAddress = sckConnect.pAddress;
				}

				// Set ip
				m_nPort = sckConnect.nPort;

				if(ASSERT_LOG(bind(m_socket, (sockaddr *)&m_sockAddr, sizeof(SOCKADDR_IN)) != SOCKET_ERROR, "Failed to bind socket."))
					return E_FAIL;

				m_socketStep = ECST_BIND;
				m_conType = ECT_CLIENT;
			}

			return S_OK;
		}

		// Load from socket informations
		RESULT CWinsock2Connection::loadFromInformation(SOCKET socket, void *ptr)
		{
			return S_OK;
		}

		// Wait for new Connect
		IConnection *CWinsock2Connection::waitConnect()
		{
			// is it server?
			if(ASSERT_LOG(m_conType != ECT_SERVER, "Wrong working with connection type."))
				return 0;

			SOCKADDR_IN sockAddr;
			SOCKET newSocket;
			c32 nSockAddrLen = sizeof(SOCKADDR_IN);

			// Waiting for accept
			newSocket = ::WSAAccept(m_socket, (SOCKADDR *)&sockAddr, &nSockAddrLen, 0, 0);
			if(newSocket == INVALID_SOCKET)
			{
				// Print error!
				io::LogPrinter::printf(io::ELL_ERROR, __FILE__, __LINE__, "WSAAccept() Error!!");
				return 0;
			}

			// Create connection object.
			IConnection *pConnection = new CWinsock2Connection(newSocket, sockAddr);

			return pConnection;
		}

		// Wait for new Connect with extended Connection object.
		IConnection *CWinsock2Connection::waitConnect(IConnection *pConnection)
		{
			// is it server?
			if(ASSERT_LOG(m_conType != ECT_SERVER, "Wrong working with connection type."))
				return 0;

			SOCKADDR_IN sockAddr;
			SOCKET newSocket;
			c32 nSockAddrLen = sizeof(SOCKADDR_IN);

			// Waiting for accept
			newSocket = ::WSAAccept(m_socket, (SOCKADDR *)&sockAddr, &nSockAddrLen, 0, 0);
			if(newSocket == INVALID_SOCKET)
			{
				// Print error!
				io::LogPrinter::printf(io::ELL_ERROR, __FILE__, __LINE__, "WSAAccept() Error!!");
				return 0;
			}

			// Set information to connection object.
			pConnection->loadFromInformation(newSocket, &sockAddr);

			return pConnection;
		}

		// Send Message.
		RESULT CWinsock2Connection::sendMessage(core::CDataBuffer **ppDataBuffer, c32 nArraySize)
		{
			core::CDataBuffer *pDataBufer = ppDataBuffer[0];
			SOCKET sckReceiver;
			WSABUF wsaBuf;

			// If Client
			if(m_conType == ECT_CLIENT)
			{
				sckReceiver = m_socket;
			}

			// Send Message
			while(pDataBufer)
			{
				wsaBuf.buf = (c8 *)pDataBufer->getPointer();
				wsaBuf.len = pDataBufer->getUsedSize();

				// Send!

				// Next!
				pDataBufer++;
			}
			return S_OK;
		}

		// Constructor
		CWinsock2Connection::CWinsock2Connection(SOCKET newSocket, SOCKADDR_IN &sockAddr)
		{
			// Set state.
			m_socketStep = ECST_CONNECT;
			m_conType = ECT_CLIENT;
			m_sockAddr = sockAddr;
			
			// Get information
			m_strAddress = inet_ntoa(sockAddr.sin_addr);
			m_nPort = ntohs(sockAddr.sin_port);
		}
	};
};

#endif