#ifndef _PURENETWORK_DEFINED_H
#define _PURENETWORK_DEFINED_H

#ifdef WIN32
#include <assert.h>
#endif

#include "IUnknown.h"
#include "LogPrinter.h"

// Defined header
//
// This file define many useful variables.

namespace pun
{
	// -----------------
	// Define Variables
	// -----------------

	// Max Address size
#define MAX_ADDRESS_SIZE 255

	// Local Address
#define LOCAL_ADDRESS "127.0.0.1"

	// Any Address
#define ANY_ADDRESS "*"

	// Invalid socket
#ifndef INVALID_SOCKET
#define INVALID_SOCKET 0
#endif
	
	// Socket Error
#ifndef SOCKET_ERROR
#define SOCKET_ERROR -1
#endif

	// -----------------
	// Define Types
	// -----------------
#ifdef SOCKET
#undef SOCKET
#endif
typedef u32 SOCKET;

	// -----------------
	// Macro Functions
	// -----------------

	// is it successed ?
#ifdef SUCCEEDED
#undef SUCCEEDED
#endif
#define SUCCEEDED(resultCode) (resultCode == 0)

	// is it failed ?
#ifdef FAILED
#undef FAILED
#endif
#define FAILED(resultCode) (resultCode != 0)

	// -----------------
	// Inline Functions
	// -----------------

	inline void SAFE_RELEASE(void *ptr)			// Safe release
	{
		if(ptr)
		{
			((IUnknown *)ptr)->release();
			ptr = 0;
		}
	}

	inline void SAFE_DELETE(void *ptr)			// Safe delete
	{
		if(ptr)
		{
			delete ptr;
			ptr = 0;
		}
	}

	inline void SAFE_DELETEARR(void *ptr)		// Safe delete for array.
	{
		if(ptr)
		{
			delete []ptr;
			ptr = 0;
		}
	}

	inline bool ASSERT_LOG(void *pValue, c8 *pLog)		// Assert Log
	{
		// Print log
		if(!pValue)
			io::LogPrinter::printf(io::ELL_FATAL_ERROR, "[ASSERT]%s", pLog);

		// Debug assert
#if (_DEBUG && WIN32)
		assert(pValue && pLog);
#endif
		
		// If Error, return true
		return (pValue == 0);
	}

	inline bool ASSERT_LOG(bool bValue, c8 *pLog)		// Assert log
	{
		// Print log
		if(!bValue)
			io::LogPrinter::printf(io::ELL_FATAL_ERROR, "[ASSERT]%s", pLog);

		// Debug assert
#if (_DEBUG && WIN32)
		assert(bValue && pLog);
#endif

		// If Error, return true
		return (bValue == false);
	}
};

#endif