#ifndef _PURENETWORK_IPURE_NETWORK_DEVICE_H
#define _PURENETWORK_IPURE_NETWORK_DEVICE_H

#include "punDataTypes.h"
#include "IEventReceiver.h"
#include "ISocket.h"

// PureNetwork Device Interface
//
// This file is for controling PureNetwork system.

namespace pun
{
	class IPureNetworkDevice : public IEventReceiver
	{
	public:

		// Constructor
		IPureNetworkDevice()
		{

		}

		// Destructor
		virtual ~IPureNetworkDevice()
		{
			// Release socket list
			core::list<socket::ISocket *>::iterator iterSocket = m_lstSocket.begin();
			for(;iterSocket != m_lstSocket.end();++iterSocket)
				SAFE_RELEASE(*iterSocket);
			m_lstSocket.clear();
		}

		// create socket
		virtual socket::ISocket *createSocket() = 0;

		// add socket
		virtual RESULT addSocket(socket::ISocket *pSocket)
		{
			// Check Parameter
			if(ASSERT_LOG(pSocket, "Wrong Parameter pSocket!"))
				return E_FAIL;

			// Refer socket.
			pSocket->ref();

			// Add in list
			m_lstSocket.push_back(pSocket);

			return S_OK;
		}

		// delete socket
		virtual RESULT deleteSocket(socket::ISocket *pSocket)
		{
			core::list<socket::ISocket *>::iterator iterSocket = m_lstSocket.find(pSocket);

			// Can not find!
			if(iterSocket == m_lstSocket.end())
				return E_FAIL;

			// Release socket.
			SAFE_RELEASE(pSocket);

			// Erase in list
			m_lstSocket.erase(iterSocket);

			// Success!
			return S_OK;
		}

		// initialize driver.
		virtual RESULT initDriver(E_NETWORK_DRIVER_TYPE driverType)
		{
#ifdef _INCLUDE_WINSOCK2
			if(driverType == ENDT_WINSOCK2)
				return S_OK;
#endif

#ifdef _INCLUDE_BSDSCK
			if(driverType == ENDT_BSDSCK)
				return S_OK;
#endif

			return false;
		}

	protected:
		
		// Socket list
		core::list<socket::ISocket *> m_lstSocket;

		// Driver type
		E_NETWORK_DRIVER_TYPE m_driverType;
	};
};

#endif