#ifndef _PURENETWORK_ISOCKET_H
#define _PURENETWORK_ISOCKET_H

#include "IEventReceiver.h"
#include "punDataTypes.h"
#include "punNetworkStructs.h"
#include "IConnection.h"
#include "IDataBuffer.h"

// Socket Interface
//
// for using Socket Systems of Engine.

namespace pun
{
	// Default using namespace
	using namespace core;

	namespace socket
	{
		// Socket class
		class ISocket : public IEventReceiver
		{
		public:
			
			// Constructor
			ISocket()
			{
				conType = ECT_SERVER;
			}

			// Destructor
			virtual ~ISocket()
			{

			}

			// Listen server.
			virtual RESULT listenServer(SckListen sckListen) = 0;

			// Connect server.
			virtual RESULT connectServer(SckConnect sckConnect) = 0;

			// Close socket.
			virtual void closeSocket() = 0;

			// Send Message
			// If Client, pReceiver = 0 means Server.
			// If Server, pReceiver = 0 means all.
			virtual RESULT sendMessage(IConnection *pReceiver, core::CDataBuffer **ppDataBuffer, c32 nArraySize = 1) = 0;

			// Set Event Receiver
			virtual void setEventReceiver(IEventReceiver *pEventReceiver) = 0;

			// Get Connection
			virtual IConnection *getConnection() = 0;

			// is Connected ?
			virtual bool isConnected() = 0;

			// is Listened ?
			virtual bool isListened() = 0;
 
		protected:

			//
			// Logical events
			//

			// New Client!
			virtual void onNewClient(IConnection *pConnection) = 0;

			// Create connection
			virtual IConnection *createConnection(IConnection **pConnection) = 0;

			// Connection type
			E_CONNECTION_TYPE conType;
		};
	};
};

#endif