#ifndef _PURENETWORK_LOGPRINTER_H
#define _PURENETWORK_LOGPRINTER_H

#include <stdio.h>
#include <string.h>
#ifdef WIN32
#include <io.h>
#endif
#ifdef LINUX
#include <sys/io.h>
#endif

// LogPrinter Global Object

namespace pun
{
	namespace io
	{
		// Define Log Level
		enum E_LOG_LEVEL
		{
			ELL_INFO = 0,				// information
			ELL_WARNING,				// warning
			ELL_ERROR,					// error, but can ignore
			ELL_FATAL_ERROR,			// error, must stop now!
			ELL_COUNT,					// Count
		};

		// Define class
		class LogPrinter
		{
		public:

			// Set file descriptor
			static void setFileDescriptor(FILE *fp)
			{
				m_nPrintTarget = fileno(stdout);
				if(!fp)
					return;

				// Set target.
				m_nPrintTarget = fileno(fp);
			}

			// Set file descriptor
			static void setFileDescriptor(c32 nTarget)
			{
				m_nPrintTarget = fileno(stdout);
				if(nTarget <= 0)
					return;

				// Set target.
				m_nPrintTarget = nTarget;
			}

			// Put log
			static void put(E_LOG_LEVEL logLevel, const c8 *pMessage);
			
			// Put log with information
			static void put(E_LOG_LEVEL logLevel, c8 *pFileName, c32 nLine, const c8 *pMessage);

			// Print log
			static void printf(E_LOG_LEVEL logLevel, c8 *pFormat...);

			// Print log with information
			static void printf(E_LOG_LEVEL logLevel, c8 *pFileName, c32 nLine, c8 *pFormat...);

			// Destroy me
			static void destroy();

		protected:

			// Check initialized data.
			static void check();

			// get string from log level types
			static c8 *getLogLevel(E_LOG_LEVEL logLevel);

			// File descriptor
			static c32 m_nPrintTarget;

			// log memory buffer.
			static c8 *m_pMemoryBuf;
		};
	};
};

#endif