#ifndef _PURENETWORK_PUN_LIST_H
#define _PURENETWORK_PUN_LIST_H

#include "punDataTypes.h"

// List Class
// this class will manage data that you want into dynamic

namespace pun
{
	namespace core
	{
		// data array list
		template <class T>
		class list
		{
		public:

			// struct for managing data
			struct SData
			{
				T Data;
				SData *pPrev;
				SData *pNext;

				// Constructor
				SData()
					: pPrev( 0 ) , pNext( 0 )
				{
				}

				// Destructor
				~SData()
				{
					if( pNext )
						delete pNext;
				}
			};

			// iterator for picking data
			class iterator
			{
			public:
				// *Constructors
				iterator()
					: m_pData(0)
				{

				}

				iterator( SData *pData )
					: m_pData( pData )
				{

				}

				// *Operators
				iterator &operator ++()
				{
					m_pData = m_pData->pNext;
					return *this;
				}

				iterator &operator --()
				{
					m_pData = m_pData->pPrev;
					return *this;
				}

				iterator operator -( c32 i )
				{
					// Wrong parameter.
					if(i > 0)
						return operator +(i);

					SData *pData = m_pData;
					for( c32 a = 0 ; a < abs(i) ; ++a )
					{
						// Can not find!
						if(!pData)
							return iterator(0);

						pData = pData->pPrev;
					}

					return iterator(pData);
				}

				iterator operator +( c32 i )
				{
					// Wrong parameter.
					if(i < 0)
						return operator -(i);

					SData *pData = m_pData;
					for( c32 a = 0 ; a < abs(i) ; ++a )
					{
						// Can not find!
						if(!pData)
							return iterator(0);

						pData = pData->pNext;
					}

					return iterator(pData);
				}

				void operator =( iterator &iter )
				{
					m_pData = iter.m_pData;
				}

				bool operator !=( iterator &iter )
				{
					return ( m_pData != iter.m_pData );
				}

				bool operator ==( iterator &iter )
				{
					return ( m_pData == iter.m_pData );
				}

				T &operator *()
				{
					return m_pData->Data;
				}

				friend class list;

			private:
				// iterator data
				SData *m_pData;
			};

			// Constructor
			list()
				: m_pRoot(0) , m_nDataCount(0)
			{
			}

			// Destructor
			~list()
			{
				clear();
			}

			// get size
			s32 size()
			{
				return m_nDataCount;
			}

			// push a data at back
			s32 push_back( const T &data )
			{
				if( !m_pRoot )
				{
					m_pRoot = new SData;
					m_pRoot->Data = data;
					m_pLast = m_pRoot;
				}
				else
				{
					m_pLast->pNext = new SData;
					m_pLast->pNext->Data = data;
					m_pLast->pNext->pPrev = m_pLast;
					m_pLast = m_pLast->pNext;
				}

				// increase count
				m_nDataCount++;

				// return data's count
				return m_nDataCount;
			}

			// push a data at front
			s32 push_front( const T &data )
			{
				if( !m_pRoot )
				{
					m_pRoot = new SData;
					m_pRoot->Data = data;
					m_pLast = m_pRoot;
				}
				else
				{
					m_pRoot->pPrev = new SData;
					m_pRoot->pPrev->Data = data;
					m_pRoot = m_pRoot->pPrev;
				}

				// increase count
				m_nDataCount++;

				// return data's count
				return m_nDataCount;
			}

			// get a begin data
			iterator begin()
			{
				return iterator( m_pRoot );
			}

			// get a end data
			// but, this function must return 'NULL' ,
			// because this function often use to loop
			iterator end()
			{
				return iterator( );
			}

			// get a last data
			iterator last()
			{
				return iterator( m_pLast );
			}

			// delete the data
			iterator erase( iterator &iter )
			{
				// datas are nothing.
				if( !m_nDataCount )
					return iterator( );

				SData *pPrev = iter.m_pData->pPrev;
				SData *pNext = iter.m_pData->pNext;

				// if data is root or last , link its next or last pointer
				if( m_pRoot == iter.m_pData )
					m_pRoot = iter.m_pData->pNext;

				if( m_pLast == iter.m_pData )
					m_pLast = iter.m_pData->pPrev;

				// delete a data struct
				iter.m_pData->pNext = 0;
				delete iter.m_pData;

				// another pointer must link each other
				if( pPrev )
					pPrev->pNext = pNext;

				if( pNext )
					pNext->pPrev = pPrev;

				m_nDataCount--;

				return iterator( pNext );
			}

			// find node by data
			iterator find( const T& data )
			{
				SData *pNode = m_pRoot;
				while( pNode )
				{
					// match each data.
					if( pNode->Data == data )
						return iterator( pNode );

					// skip data
					pNode = pNode->pNext;
				}

				// can't find
				return iterator( 0 );
			}

			// get data by count
			iterator get(c32 c)
			{
				SData *pNode = m_pRoot;
				c32 i = 0;

				while( pNode )
				{
					// match each data.
					if(i == c)
						return iterator(pNode);

					// skip data
					pNode = pNode->pNext;
					++i;
				}

				// can't find
				return iterator(0);
			}

			// clear datas
			void clear()
			{
				if( m_pRoot )
					// delete data
					delete m_pRoot;

				m_pRoot = 0;
				m_pLast = 0;

				m_nDataCount = 0;
			}

			// if list is empty ?
			bool empty()
			{
				return ( !m_nDataCount );
			}

		private:
			// root data
			SData *m_pRoot;

			// last data
			SData *m_pLast;

			// data count
			u32 m_nDataCount;
		};
	};
};

#endif