#ifndef _PURENETWORK_PUN_NETWORK_STRUCTS_H
#define _PURENETWORK_PUN_NETWORK_STRUCTS_H

#include "punDataTypes.h"
#include "Defined.h"
#include "punString.h"

// Network Structures header
//
// This file has many structures to use on network system.

namespace pun
{
	// Network library type
	enum E_NETWORK_DRIVER_TYPE
	{
		// Winsock2
		ENDT_WINSOCK2 = 0,

		// BSD Socket
		ENDT__BSD,
	};

	// Address type
	enum E_ADDRESS_TYPE
	{
		// IP address version 4.
		EAT_IPV4 = 0,

		// IP address version 6.
		EAT_IPV6,

		// Domain
		EAT_DOMAIN,

		// Count
		EAT_COUNT,
	};

	// Protocol type
	enum E_PROTOCOL_TYPE
	{
		// TCP
		EPT_TCP = 0,

		// UDP
		EPT_UDP,

		// Count
		EPT_COUNT,
	};

	// Connection type
	enum E_CONNECTION_TYPE
	{
		// Server
		ECT_SERVER = 0,
		
		// Client
		ECT_CLIENT,

		// Unknown
		ECT_UNKNOWN,

		// Count
		ECT_COUNT,
	};

	// Socket type
	enum E_CONNECTION_SOCKET_TYPE
	{
		// Step 0. Empty
		ECST_EMPTY = 0,

		// Step 1. Create
		ECST_CREATE,

		// Step 2. Bind
		ECST_BIND,

		// (Only Server)
		// Step 3. Listen
		ECST_COMPLETE,

		// (Only Client)
		// Step 3. Connect
		ECST_CONNECT,
	};

	// Thread type
	enum E_THREAD_TYPE
	{
		// For accept
		ETT_ACCEPT = 0,

		// For work
		ETT_WORK,

		// Etc.
		ETT_ETC,

		// Count
		ETT_COUNT
	};

	// Thread state
	enum E_THREAD_STATE
	{
		// Ready!
		ETS_READY = 0,

		// Working
		ETS_WORKING,

		// Unknown
		ETS_UNKNOWN,

		// Count
		ETS_COUNT
	};

	// Event Message Type
	enum E_EVENT_MESSAGE_TYPE
	{
		// Socket event.
		EEMT_SOCKET = 0,
	};

	// Event Connection Message Code
	enum E_CONNECTION_MESSAGE_TYPE
	{
		// New Client!
		ECMT_NEW_CLIENT = 0,
	};

	// Party structure
	struct SckParty
	{
		// Party name.
		core::stringc strPartyName;

		// Max connection.
		// -1 means No-limited.
		// 0 means zero.
		c32 m_nMaxConnection;

		// 
	};

	// Listen structure
	struct SckListen
	{
		// Address Type
		// if EAT_ANY is checked, Connection can connect wherever.
		E_ADDRESS_TYPE addressType;

		// Protocol Type
		E_PROTOCOL_TYPE protocolType;

		// Address
		c8 pAddress[MAX_ADDRESS_SIZE];

		// Max connection
		c32 nMaxConnection;

		// Port(Only use 0-65536)
		c32 nPort;

		// Backlog
		c32 nBacklog;

		// Overlapped socket(for Winsock2)
		bool bOverlapped;

		// Constructor
		SckListen()
		{
			addressType = EAT_IPV4;
			protocolType = EPT_TCP;
			bOverlapped = false;
			nMaxConnection = 32;
			nPort = 0;
			pAddress[0] = '\0';
			nBacklog = 32;
		}
	};

	// Connect structure
	struct SckConnect
	{
		// Address Type
		E_ADDRESS_TYPE addressType;

		// Protocol Type
		E_PROTOCOL_TYPE protocolType;

		// Address
		c8 pAddress[MAX_ADDRESS_SIZE];

		// Port(Only use 0-65536)
		c32 nPort;

		// Overlapped socket(for Winsock2)
		bool bOverlapped;

		// Constructor
		SckConnect()
		{
			addressType = EAT_IPV4;
			protocolType = EPT_TCP;
			bOverlapped = false;
			nPort = 0;
			pAddress[0] = '\0';
		}

	};
};

#endif