#ifndef _PURENETWORK_PUN_STRING_H
#define _PURENETWORK_PUN_STRING_H

#include <string.h>
#include "punDataTypes.h"

// String Class
// this class will manage string buffers.

namespace pun
{
	namespace core
	{
		template <class T>
		class string
		{
		public:
			// *Constructors

			// reset buffer to null
			string()
				: m_nBufferLen( 1 ) , m_nStringLen( 0 )
			{
				// allocate null value buffer
				m_pBuffer = new c8[ 1 ];
				m_pBuffer[ 0 ] = '\0';
			}

			// reset buffer to charactor string
			string( T *pString )
				: m_pBuffer(0) , m_nBufferLen( 0 )
			{
				setString( pString );
			}

			// reset buffer to string object
			string( string<T> &str )
				: m_pBuffer(0) , m_nBufferLen( 0 )
			{
				setString( str );
			}

			// Destructor
			virtual ~string()
			{
				if( m_pBuffer )
					delete []m_pBuffer;
			}

			// get string length
			s32 getStrLength()
			{
				return m_nStringLen;
			}

			// get buffer length
			s32 getBufLength()
			{
				return m_nBufferLen;
			}

			// set text to charactor string pointer
			string<T> &setString( T *pString )
			{
				// put length data
				if( pString )
					m_nBufferLen = (s32)strlen( pString ) + 1;
				else
					m_nBufferLen = 1;
				m_nStringLen = m_nBufferLen - 1;

				// allocate string buffer and copy it.
				T *pNewData = new T[ m_nBufferLen ];
				if( pString )
					strncpy( pNewData , pString , m_nStringLen );
				pNewData[ m_nStringLen ] = '\0';

				// delete old buffer
				if( m_pBuffer )
					delete []m_pBuffer;

				m_pBuffer = pNewData;

				return *this;
			}

			// set text to string object
			string<T> &setString( string<T> &str )
			{
				// put length data
				m_nBufferLen = str.getBufLength();
				m_nStringLen = str.getStrLength();

				// allocate string buffer and copy it.
				T *pNewData = new T[ m_nBufferLen ];
				strncpy( pNewData , str.getString() , m_nStringLen );	
				pNewData[ m_nStringLen ] = '\0';

				// delete old buffer
				if( m_pBuffer )
					delete []m_pBuffer;

				m_pBuffer = pNewData;

				return *this;
			}

			// get text buffer
			T *getString()
			{
				return m_pBuffer;
			}

			// overriding type
			operator T*()
			{
				return m_pBuffer;
			}

			// *Operators

			// reset buffer to charactor string
			string<T> &operator =( T *pString )
			{
				setString( pString );

				return *this;
			}

			// reset buffer to string object
			string<T> &operator =( string<T> &str )
			{
				setString( str );

				return *this;
			}

			// add new string to old buffer
			string<T> operator +( T *pString )
			{
				core::stringc strNew(*this);
				// check parameter
				if( !pString )
					return strNew;

				strNew += pString;

				return strNew;
			}

			// add new string to old buffer
			string<T> operator +( string<T> &str )
			{
				core::stringc strNew(*this);
				// check parameter
				if(!str.getStrLength())
					return strNew;
				strNew += str;

				return strNew;
			}

			// check old string equal to parameter string
			bool operator ==( T *pString )
			{
				if( pString == NULL )
				{
					// if string to compare is null, check old string is null
					if( !m_nStringLen )
						return true;

					// else
					return false;
				}

				u32 nNewBufLen = (u32)strlen( pString );

				// buffer's length is zero
				if( !nNewBufLen )
					return false;

				// buffer's length is not qqual
				if( nNewBufLen != m_nStringLen )
					return false;

				// check each other's buffer
				for( u32 i = 0 ; i < (u32)m_nStringLen ; i ++ )
				{
					if( pString[ i ] != m_pBuffer[ i ] )
						return false;
				}

				return true;
			}

			// check old string is equal to parameter string
			bool operator ==( string<T> &str )
			{
				if( !str.getStrLength() )
				{
					// if string to compare is null, check old string is null
					if( !m_nStringLen )
						return true;

					// else
					return false;
				}

				u32 nNewBufLen = str.getStrLength();

				// buffer's length is zero
				if( !nNewBufLen )
					return false;

				// buffer's length is not qqual
				if( nNewBufLen != m_nStringLen )
					return false;

				T *pStr = str.getString();

				// check each other's buffer
				for( u32 i = 0 ; i < (u32)m_nStringLen ; i ++ )
				{
					if( pStr[ i ] != m_pBuffer[ i ] )
						return false;
				}

				return true;
			}

			// check old string is not equal to parameter string
			bool operator !=( T *pString )
			{
				return !( operator==( pString ) );
			}

			// check old string is not equal to parameter string
			bool operator !=( string<T> &str )
			{
				return !( operator==( str ) );
			}

			// add string
			string<T> &operator +=( T *pString )
			{
				if( !m_nStringLen )
				{
					setString( pString );
					return *this;
				}

				u32 nStrLen = m_nStringLen + (u32)strlen(pString);

				T *pNewData = new T[ nStrLen + 1 ];
				memcpy( pNewData , m_pBuffer , m_nStringLen );
				strncpy( &pNewData[m_nStringLen] , pString , strlen(pString) );
				pNewData[ nStrLen ] = '\0';

				// release and refresh
				if( m_pBuffer )
					delete []m_pBuffer;
				m_pBuffer = pNewData;

				m_nStringLen = nStrLen;
				m_nBufferLen = nStrLen + 1;
				return *this;
			}

			// add string
			string<T> &operator +=( string<T> &str )
			{
				if( !m_nStringLen )
				{
					setString( str );
					return *this;
				}

				u32 nStrLen = m_nStringLen + str.getStrLength();

				// alloc buffer
				T *pNewData = new T[ nStrLen + 1 ];

				// copy string
				memcpy( pNewData , m_pBuffer , m_nStringLen );
				strncpy( &pNewData[m_nStringLen] , str.getString() , str.getStrLength() );
				pNewData[ nStrLen ] = '\0';

				// release and refresh
				if( m_pBuffer )
					delete []m_pBuffer;
				m_pBuffer = pNewData;

				m_nStringLen = nStrLen;
				m_nBufferLen = nStrLen + 1;

				return *this;
			}

			// clear buffer
			void setClear()
			{
				// delete old buffer
				if( m_pBuffer )
					delete []m_pBuffer;

				m_pBuffer = NULL;

				m_nBufferLen = 0;
				m_nStringLen = 0;
			}

			// find string
			T *findStr( const T *pToken )
			{
				u16 nLen = strlen(pToken);
				for( s32 i = 0 ; i < m_nStringLen - (s32)nLen ; i ++ )
				{
					if( !strnicmp( &m_pBuffer[i] , pToken , nLen ) )
						return &m_pBuffer[i];
				}
				return 0;
			}

			// find string and return count
			s32 findStrCount( const T *pToken )
			{
				u16 nLen = strlen(pToken);
				for( s32 i = 0 ; i < m_nStringLen - (s32)nLen ; i ++ )
				{
					if( !strnicmp( &m_pBuffer[i] , pToken , nLen ) )
						return i;
				}
				return -1;
			}

			// find string reverse
			T *findStrReverse( const T *pToken )
			{
				for( s32 i = m_nStringLen - (s32)strlen(pToken) - 1 ; i >= 0  ; i -- )
				{
					if( strstr( &m_pBuffer[i] , pToken ) )
						return &m_pBuffer[i];
				}
				return 0;
			}

			// trim left side
			string<T> trimLeft()
			{
				string<T> str;
				u32 i = 0;
				while( m_pBuffer[i] == '\t' || m_pBuffer[i] == ' ' || m_pBuffer[i] == '\0' )
					i++;

				str.setString( &m_pBuffer[i] );

				return str;
			}

			// trim all side
			string<T> trim()
			{
				string<T> str(*this);
				T *p = str.getString();
				u32 nDel = 0;

				for( s32 i = 0 ; i < m_nStringLen ; i ++ )
					if( m_pBuffer[i] != '\t' && m_pBuffer[i] != ' ' )
						p[nDel++] = m_pBuffer[i];

				p[nDel] = '\0';
				str.m_nStringLen = nDel;

				// adjust buffer
				str.adjust();

				return str;
			}

			// go to the next string myself
			string<T> &goNextString()
			{
				// alloc new buffer
				T *p = new T[ m_nStringLen + 1 ];

				bool bSpace = false;
				bool bSpace2 = false;
				u32 nDel = 0 ;

				for( s32 i = 0 ; i < m_nStringLen ; i ++ )
				{
					if( !bSpace && ( m_pBuffer[i] == '\t' || m_pBuffer[i] == ' ' ) )
						bSpace = true;
					else if( bSpace )
					{
						if( !bSpace2 && !(m_pBuffer[i] == '\t' || m_pBuffer[i] == ' ' ) )
							bSpace2 = true;

						if(bSpace2)
						{
							// write data
							p[nDel++] = m_pBuffer[i];
						}
					}
				}

				// put null character
				p[ nDel ] = '\0';

				// change into new buffer
				if( m_pBuffer )
					delete []m_pBuffer;

				m_pBuffer = p;
				m_nStringLen = nDel;

				// adjust buffer
				adjust();

				return *this;
			}

			// adjust buffer
			string<T> &adjust()
			{
				T *p = new T[ m_nStringLen + 1 ];
				memcpy( p , m_pBuffer , m_nStringLen );
				p[ m_nStringLen ] = '\0';

				if( m_pBuffer )
					delete []m_pBuffer;

				m_pBuffer = p;

				// reset information
				m_nBufferLen = m_nStringLen + 1;

				return *this;
			}

			// check to match string
			bool match( T *pString , s32 nLength = -1 )
			{
				// get buffer's length
				if( nLength == -1 )
					nLength = (s32)strlen(pString);

				T *p = pString;

				// check to match
				for( s32 i = 0 ; i < nLength ; i ++ )
				{
					if( p[i] != m_pBuffer[i] )
						return false;
				}

				return true;
			}

			// check to match string
			bool match( string<T> &str , s32 nLength = -1 )
			{
				// get buffer's length
				if( nLength == -1 )
					nLength = str.getStrLength();

				T *p = str.getString();

				// check to match
				for( u32 i = 0 ; i < nLength ; i ++ )
				{
					if( p[i] != m_pBuffer[i] )
						return false;
				}

				return true;
			}

			// cut string to want by left
			string<T> left( s32 nCount )
			{
				core::stringc strNew = *this;

				// check length
				if( m_nStringLen <= nCount || nCount < 1)
					return strNew;

				strNew->m_pBuffer[ nCount ] = '\0';
				strNew->m_nStringLen = nCount;
				strNew->adjust();

				return strNew;
			}

		private:
			// string buffer
			T *m_pBuffer;

			// buffer length
			s32 m_nBufferLen;

			// string length
			s32 m_nStringLen;
		};

		// 8bits string
		typedef string<c8> stringc;

		// wide string
		typedef string<wchar_t> stringw;
	};
};

#endif